import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:videosdk/events.dart';
import 'package:videosdk/rtc.dart';
import 'package:videosdk/src/features/me/bloc/me_bloc.dart';
import 'package:videosdk/src/features/peers/enitity/peer.dart';
import 'package:videosdk/src/features/room/bloc/room_bloc.dart';
import 'package:videosdk/src/features/signaling/room_client_repository.dart';
import 'package:events2/events2.dart';
import 'dart:io' show Platform;
import 'package:videosdk/src/features/pubsub/pubsub.dart';

class Meeting {
  final BuildContext context;
  late final EventEmitter _eventEmitter, _topicEventEmitter;

  Meeting(this.context) {
    this.localParticipant = new Participant(
      context,
      context.read<MeBloc>().state.id,
      context.read<MeBloc>().state.displayName,
      true,
    );

    this._eventEmitter = context.read<RoomClientRepository>().eventEmitter;
    _topicEventEmitter = EventEmitter();

    this.participants = new Map();

    context.read<RoomClientRepository>().onMeetingJoined =
        (roomClientRepository) {
      // on meeting joined
    };

    _eventEmitter.on("peers-bloc-participant-joined", (Peer newPeer) {
      final newParticipant = new Participant(
        context,
        newPeer.id,
        newPeer.displayName,
        false,
      );
      participants[newPeer.id] = newParticipant;
      _eventEmitter.emit('participant-joined', newParticipant);
    });

    _eventEmitter.on("peers-bloc-participant-left", (peerId) {
      participants.remove(peerId);
      _eventEmitter.emit('participant-left', peerId);
    });

    _eventEmitter.on("pubsub-message", this.handlePubSubMessage);

    _eventEmitter.on("peers-bloc-presenter-changed", (peerId) {
      context
          .read<RoomBloc>()
          .add(RoomSetActivePresenterId(presenterId: peerId));
    });

    pubSub = PubSub(
        roomClient: context.read<RoomClientRepository>(),
        topicEventEmitter: _topicEventEmitter);
  }

  late Participant localParticipant;

  late Map<String, Participant> participants;

  late PubSub pubSub;

  String? get activeSpeakerId {
    return context.read<RoomBloc>().state.activeSpeakerId;
  }

  String? get activePresenterId {
    return context.read<RoomBloc>().state.activePresenterId;
  }

  void disableWebcam() {
    context.read<RoomClientRepository>().disableWebcam();
  }

  void enableWebcam() {
    context.read<RoomClientRepository>().enableWebcam();
  }

  void muteMic() {
    context.read<RoomClientRepository>().disableMic();
  }

  void unmuteMic() {
    context.read<RoomClientRepository>().enableMic();
  }

  void enableScreenShare() {
    if (Platform.isAndroid) {
      context.read<RoomClientRepository>().enableShare();
    }
  }

  void disableScreenShare() {
    if (Platform.isAndroid) {
      context.read<RoomClientRepository>().disableShare();
    }
  }

  void join() {
    context.read<RoomClientRepository>().join();
  }

  void leave() {
    context.read<RoomClientRepository>().close();
  }

  void startRecording(String webhookUrl) {
    context.read<RoomClientRepository>().startRecording(webhookUrl);
  }

  void stopRecording() {
    context.read<RoomClientRepository>().stopRecording();
  }

  void startLivestream(outputs) {
    context.read<RoomClientRepository>().startLivestream(outputs);
  }

  void stopLivestream() {
    context.read<RoomClientRepository>().stopLivestream();
  }

  void changeWebcam(String deviceId) {
    context.read<RoomClientRepository>().changeWebcam(deviceId);
  }

  List<MediaDeviceInfo> getWebcams() {
    return context.read<RoomClientRepository>().getWebcams();
  }

  String? get selectedWebcamId {
    return context.read<RoomClientRepository>().selectedWebcamId;
  }

  List<MediaDeviceInfo> getMics() {
    return context.read<RoomClientRepository>().getMics();
  }

  String? get selectedMicId {
    return context.read<RoomClientRepository>().selectedMicId;
  }

  void handlePubSubMessage(data) {
    _topicEventEmitter.emit(
      data['topic'],
      PubSubMessage.fromJson(data),
    );
  }

  on(Events event, Function handler) {
    this._eventEmitter.on(event.parseToString(), handler);
  }
}
