import 'dart:async';
import 'package:collection/collection.dart';

import 'package:bloc/bloc.dart';
import 'package:equatable/equatable.dart';
import 'package:videosdk/src/features/peers/enitity/peer.dart';
import 'package:videosdk/src/features/media_devices/bloc/media_devices_bloc.dart';
// import 'package:flutter/foundation.dart';
import 'package:flutter_webrtc/flutter_webrtc.dart';
import './../../../videosdk_client/videosdk_client.dart';
import 'package:events2/events2.dart';

part 'peers_event.dart';
part 'peers_state.dart';

class PeersBloc extends Bloc<dynamic, PeersState> {
  late final EventEmitter eventEmitter;
  final MediaDevicesBloc mediaDevicesBloc;
  String selectedOutputId = '';

  PeersBloc({required this.mediaDevicesBloc, required this.eventEmitter})
      : super(PeersState()) {
    // if (mediaDevicesBloc.state.selectedAudioOutput?.deviceId != null) {
    //   selectedOutputId = mediaDevicesBloc.state.selectedAudioOutput!.deviceId;
    // }
    //
    // mediaDevicesBloc.stream.listen((event) {
    //   state.peers.values.forEach((p) {
    //     final String? deviceId = event.selectedAudioOutput?.deviceId;
    //     if (deviceId != null) {
    //       selectedOutputId = deviceId;
    //       p.renderer?.audioOutput = selectedOutputId;
    //     }
    //   });
    // });
  }

  @override
  Stream<PeersState> mapEventToState(
    dynamic event,
  ) async* {
    if (event is PeerAdd) {
      yield* _mapPeerAddToState(event);
    } else if (event is PeerRemove) {
      yield* _mapPeerRemoveToState(event);
    } else if (event is PeerAddConsumer) {
      yield* _mapConsumerAddToState(event);
    } else if (event is PeerRemoveConsumer) {
      yield* _mapConsumerRemoveToState(event);
    } else if (event is PeerPausedConsumer) {
      yield* _mapPeerPausedConsumer(event);
    } else if (event is PeerResumedConsumer) {
      yield* _mapPeerResumedConsumer(event);
    } else if (event is PeerChangeConsumerQuality) {
      yield* _mapPeerChangeConsumerQuality(event);
    }
  }

  Stream<PeersState> _mapPeerAddToState(PeerAdd event) async* {
    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    final Peer newPeer = Peer.fromMap(event.newPeer);
    newPeers[newPeer.id] = newPeer;

    yield PeersState(peers: newPeers);

    this.eventEmitter.emit("peers-bloc-participant-joined", newPeer);
  }

  Stream<PeersState> _mapPeerRemoveToState(PeerRemove event) async* {
    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    newPeers.remove(event.peerId);

    yield PeersState(peers: newPeers);

    this.eventEmitter.emit("peers-bloc-participant-left", event.peerId);
  }

  Stream<PeersState> _mapConsumerAddToState(PeerAddConsumer event) async* {
    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);

    // if (kIsWeb) {
    //   if (newPeers[event.peerId]!.renderer == null) {
    //     newPeers[event.peerId] =
    //         newPeers[event.peerId]!.copyWith(renderer: RTCVideoRenderer());
    //     await newPeers[event.peerId]!.renderer!.initialize();
    //     // newPeers[event.peerId]!.renderer!.audioOutput = selectedOutputId;
    //   }

    //   if (event.consumer.kind == 'video') {
    //     newPeers[event.peerId] =
    //         newPeers[event.peerId]!.copyWith(video: event.consumer);
    //     newPeers[event.peerId]!.renderer!.srcObject =
    //         newPeers[event.peerId]!.video!.stream;
    //   }

    //   if (event.consumer.kind == 'audio') {
    //     newPeers[event.peerId] =
    //         newPeers[event.peerId]!.copyWith(audio: event.consumer);
    //     if (newPeers[event.peerId]!.video == null) {
    //       newPeers[event.peerId]!.renderer!.srcObject =
    //           newPeers[event.peerId]!.audio!.stream;
    //     }
    //   }
    // } else {
    if (event.consumer.kind == 'video' &&
        event.consumer.appData['share'] == true) {
      //

      newPeers[event.peerId] = newPeers[event.peerId]!.copyWith(
        shareRenderer: RTCVideoRenderer(),
        share: event.consumer,
      );

      await newPeers[event.peerId]!.shareRenderer!.initialize();

      newPeers[event.peerId]!.shareRenderer!.srcObject =
          newPeers[event.peerId]!.share!.stream;

      this.eventEmitter.emit("stream-enabled-${event.peerId}", {
        "shareRenderer": newPeers[event.peerId]!.shareRenderer,
        "share": newPeers[event.peerId]!.share
      });

      this.eventEmitter.emit("peers-bloc-presenter-changed", event.peerId);

      //
    } else if (event.consumer.kind == 'video') {
      //
      newPeers[event.peerId] = newPeers[event.peerId]!.copyWith(
        renderer: RTCVideoRenderer(),
        video: event.consumer,
      );

      await newPeers[event.peerId]!.renderer!.initialize();

      newPeers[event.peerId]!.renderer!.srcObject =
          newPeers[event.peerId]!.video!.stream;

      this.eventEmitter.emit("stream-enabled-${event.peerId}", {
        "renderer": newPeers[event.peerId]!.renderer,
        "video": newPeers[event.peerId]!.video
      });
      //
    } else if (event.consumer.kind == 'audio') {
      newPeers[event.peerId] = newPeers[event.peerId]!.copyWith(
        audio: event.consumer,
      );

      this.eventEmitter.emit("stream-enabled-${event.peerId}", {
        "audio": event.consumer,
      });
    }
    // }

    yield PeersState(peers: newPeers);
  }

  Stream<PeersState> _mapConsumerRemoveToState(
      PeerRemoveConsumer event) async* {
    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    final Peer? peer = newPeers.values
        .firstWhereOrNull((p) => p.consumers.contains(event.consumerId));

    if (peer != null) {
      // if (kIsWeb) {
      //   if (peer.audio?.id == event.consumerId) {
      //     final consumer = peer.audio;
      //     if (peer.video == null) {
      //       final renderer = newPeers[peer.id]?.renderer!;
      //       newPeers[peer.id] = newPeers[peer.id]!.removeAudioAndRenderer();
      //       yield PeersState(peers: newPeers);
      //       await renderer?.dispose();
      //     } else {
      //       newPeers[peer.id] = newPeers[peer.id]!.removeAudio();
      //       yield PeersState(peers: newPeers);
      //     }
      //     await consumer?.close();
      //   } else if (peer.video?.id == event.consumerId) {
      //     final consumer = peer.audio;
      //     if (peer.audio != null) {
      //       newPeers[peer.id]!.renderer!.srcObject =
      //           newPeers[peer.id]!.audio!.stream;
      //       newPeers[peer.id] = newPeers[peer.id]!.removeVideo();
      //       yield PeersState(peers: newPeers);
      //     } else {
      //       final renderer = newPeers[peer.id]!.renderer!;
      //       newPeers[peer.id] = newPeers[peer.id]!.removeVideoAndRenderer();
      //       yield PeersState(peers: newPeers);
      //       await renderer.dispose();
      //     }
      //     await consumer?.close();
      //   }
      // } else {
      if (peer.audio?.id == event.consumerId) {
        final consumer = peer.audio;
        newPeers[peer.id] = newPeers[peer.id]!.removeAudio();

        await consumer?.close();

        this.eventEmitter.emit(
          "stream-disabled-${peer.id}",
          {
            "audioConsumerId": consumer?.id,
          },
        );

        yield PeersState(peers: newPeers);
      } else if (peer.video?.id == event.consumerId) {
        final consumer = peer.video;
        final renderer = peer.renderer;
        newPeers[peer.id] = newPeers[peer.id]!.removeVideoAndRenderer();

        await consumer?.close();

        await renderer?.dispose();

        this.eventEmitter.emit(
          "stream-disabled-${peer.id}",
          {
            "renderer": renderer,
            "videoConsumerId": consumer?.id,
          },
        );

        yield PeersState(peers: newPeers);
      } else if (peer.share?.id == event.consumerId) {
        final consumer = peer.share;
        final renderer = peer.shareRenderer;

        newPeers[peer.id] = newPeers[peer.id]!.removeShareAndRenderer();

        await consumer?.close();

        await renderer?.dispose();

        this.eventEmitter.emit(
          "stream-disabled-${peer.id}",
          {
            "shareRenderer": renderer,
            "shareConsumerId": consumer?.id,
          },
        );

        this.eventEmitter.emit("peers-bloc-presenter-changed");

        yield PeersState(peers: newPeers);
      }
      // }
    }
  }

  Stream<PeersState> _mapPeerPausedConsumer(PeerPausedConsumer event) async* {
    String consumerId = event.consumerId;

    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    final Peer? peer = newPeers.values
        .firstWhereOrNull((p) => p.consumers.contains(consumerId));

    if (peer != null) {
      if (peer.audio?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          audio: peer.audio!.pauseCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-paused-${peer.id}", {
          "audioConsumerId": consumerId,
          "consumer": newPeer.audio,
        });
      } else if (peer.video?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          video: peer.video!.pauseCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-paused-${peer.id}", {
          "videoConsumerId": consumerId,
          "consumer": newPeer.video,
        });
      } else if (peer.share?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          share: peer.share!.pauseCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-paused-${peer.id}", {
          "shareConsumerId": consumerId,
          "consumer": newPeer.share,
        });
      }

      yield PeersState(peers: newPeers);
    }
  }

  Stream<PeersState> _mapPeerResumedConsumer(PeerResumedConsumer event) async* {
    String consumerId = event.consumerId;

    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    final Peer? peer = newPeers.values
        .firstWhereOrNull((p) => p.consumers.contains(consumerId));

    if (peer != null) {
      if (peer.audio?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          audio: peer.audio!.resumeCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-resumed-${peer.id}", {
          "audioConsumerId": consumerId,
          "consumer": newPeer.audio,
        });
      } else if (peer.video?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          video: peer.video!.resumeCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-resumed-${peer.id}", {
          "videoConsumerId": consumerId,
          "consumer": newPeer.video,
        });
      } else if (peer.share?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
          share: peer.share!.resumeCopy(),
        );

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("stream-resumed-${peer.id}", {
          "shareConsumerId": consumerId,
          "consumer": newPeer.share,
        });
      }

      yield PeersState(peers: newPeers);
    }
  }

  Stream<PeersState> _mapPeerChangeConsumerQuality(
      PeerChangeConsumerQuality event) async* {
    String consumerId = event.consumerId;
    int spatialLayer = event.spatialLayer;
    int temporalLayer = event.temporalLayer;

    final Map<String, Peer> newPeers = Map<String, Peer>.of(state.peers);
    final Peer? peer = newPeers.values
        .firstWhereOrNull((p) => p.consumers.contains(consumerId));

    if (peer != null) {
      if (peer.audio?.id == consumerId) {
      } else if (peer.video?.id == consumerId) {
        Peer newPeer = newPeers[peer.id]!.copyWith(
            video: peer.video!.copyWith(
          spatialLayer: spatialLayer,
          temporalLayer: temporalLayer,
        ));

        newPeers[peer.id] = newPeer;

        this.eventEmitter.emit("quality-changed-${peer.id}", {
          "spatialLayer": spatialLayer,
          "temporalLayer": temporalLayer,
          "videoConsumerId": consumerId
        });
      } else if (peer.share?.id == consumerId) {}

      yield PeersState(peers: newPeers);
    }
  }
}
