import 'dart:developer';

import 'package:events2/events2.dart';
import 'package:videosdk/src/features/pubsub/pubsub_message.dart';
import '../signaling/room_client_repository.dart';
import 'package:synchronized/synchronized.dart';

import 'pubsub_publish_options.dart';

class PubSub {
  RoomClientRepository? _roomClient;
  EventEmitter? _topicEventEmitter;
  late final Lock lock;

  PubSub({required roomClient, required topicEventEmitter}) {
    this._roomClient = roomClient;
    this._topicEventEmitter = topicEventEmitter;

    this.lock = new Lock();
  }

  Future<void> publish(String topic, String message,
      [PubSubPublishOptions options = const PubSubPublishOptions()]) async {
    await _roomClient!.pubsubPublish(
      topic: topic,
      message: message,
      options: options.toJson(),
    );
  }

  Future<PubSubMessages> subscribe(
          String topic, Function(PubSubMessage) onMessageReceived) =>
      lock.synchronized(() async {
        final msgs = (await _roomClient!.pubsubSubscribe(topic)) ??
            <String, dynamic>{"messages": []};

        var messages = PubSubMessages.fromJson(msgs);

        if (!_topicEventEmitter!.listeners(topic).contains(onMessageReceived)) {
          _topicEventEmitter!.on(topic, onMessageReceived);
        }

        return messages;
      });

  Future<void> unsubscribe(
      String topic, Function(PubSubMessage) onMessageReceived) async {
    try {
      _topicEventEmitter!.remove(topic, onMessageReceived);
    } catch (e) {
      log("unsubscribe() | error: $e");
    }

    if (_topicEventEmitter!.listeners(topic).length == 0) {
      await _roomClient!.pubsubUnsubscribe(topic);
    }
  }
}
